use actix_web::{web, HttpResponse, Responder, Result};
use actix_session::Session;
use tera::Tera;
use serde_json::Value;

/// Controller for handling home-related routes
pub struct HomeController;

impl HomeController {
    /// Helper function to get user from session
    fn get_user_from_session(session: &Session) -> Option<Value> {
        session.get::<String>("user").ok().flatten().and_then(|user_json| {
            serde_json::from_str(&user_json).ok()
        })
    }

    /// Handles the markdown editor page route
    pub async fn editor(tmpl: web::Data<Tera>, session: Session) -> Result<impl Responder> {
        let mut ctx = tera::Context::new();
        ctx.insert("active_page", "editor");
        
        // Add user to context if available
        if let Some(user) = Self::get_user_from_session(&session) {
            ctx.insert("user", &user);
        }
        
        let rendered = tmpl.render("editor.html", &ctx)
            .map_err(|e| {
                eprintln!("Template rendering error: {}", e);
                actix_web::error::ErrorInternalServerError("Template rendering error")
            })?;
        
        Ok(HttpResponse::Ok().content_type("text/html").body(rendered))
    }

    /// Handles the home page route
    pub async fn index(tmpl: web::Data<Tera>, session: Session) -> Result<impl Responder> {
        let mut ctx = tera::Context::new();
        ctx.insert("active_page", "home");
        
        // Add user to context if available
        if let Some(user) = Self::get_user_from_session(&session) {
            ctx.insert("user", &user);
        }
        
        let rendered = tmpl.render("index.html", &ctx)
            .map_err(|e| {
                eprintln!("Template rendering error: {}", e);
                actix_web::error::ErrorInternalServerError("Template rendering error")
            })?;
        
        Ok(HttpResponse::Ok().content_type("text/html").body(rendered))
    }
    
    /// Handles the about page route
    pub async fn about(tmpl: web::Data<Tera>, session: Session) -> Result<impl Responder> {
        let mut ctx = tera::Context::new();
        ctx.insert("active_page", "about");
        
        // Add user to context if available
        if let Some(user) = Self::get_user_from_session(&session) {
            ctx.insert("user", &user);
        }
        
        let rendered = tmpl.render("about.html", &ctx)
            .map_err(|e| {
                eprintln!("Template rendering error: {}", e);
                actix_web::error::ErrorInternalServerError("Template rendering error")
            })?;
        
        Ok(HttpResponse::Ok().content_type("text/html").body(rendered))
    }
    
    /// Handles the contact page route
    pub async fn contact(tmpl: web::Data<Tera>, session: Session) -> Result<impl Responder> {
        let mut ctx = tera::Context::new();
        ctx.insert("active_page", "contact");
        
        // Add user to context if available
        if let Some(user) = Self::get_user_from_session(&session) {
            ctx.insert("user", &user);
        }
        
        let rendered = tmpl.render("contact.html", &ctx)
            .map_err(|e| {
                eprintln!("Template rendering error: {}", e);
                actix_web::error::ErrorInternalServerError("Template rendering error")
            })?;
        
        Ok(HttpResponse::Ok().content_type("text/html").body(rendered))
    }
    
    /// Handles form submissions from the contact page
    pub async fn submit_contact(
        form: web::Form<ContactForm>,
        tmpl: web::Data<Tera>,
        session: Session
    ) -> Result<impl Responder> {
        // In a real application, you would process the form data here
        // For example, save it to a database or send an email
        
        println!("Contact form submission: {:?}", form);
        
        // For this example, we'll just redirect back to the contact page
        // with a success message
        let mut ctx = tera::Context::new();
        ctx.insert("active_page", "contact");
        ctx.insert("success_message", "Your message has been sent successfully!");
        
        // Add user to context if available
        if let Some(user) = Self::get_user_from_session(&session) {
            ctx.insert("user", &user);
        }
        
        let rendered = tmpl.render("contact.html", &ctx)
            .map_err(|e| {
                eprintln!("Template rendering error: {}", e);
                actix_web::error::ErrorInternalServerError("Template rendering error")
            })?;
        
        Ok(HttpResponse::Ok().content_type("text/html").body(rendered))
    }
}

/// Represents the data submitted in the contact form
#[derive(Debug, serde::Deserialize)]
pub struct ContactForm {
    pub name: String,
    pub email: String,
    pub subject: String,
    pub message: String,
}