use actix_web::{web, HttpResponse, Responder, Result};
use actix_session::Session;
use tera::Tera;
use serde::{Deserialize, Serialize};
use serde_json::Value;
use crate::models::{User, Ticket, TicketComment, TicketStatus, TicketPriority};
use std::collections::HashMap;
use std::sync::{Arc, Mutex};

// In-memory storage for tickets and comments (in a real app, this would be a database)
lazy_static::lazy_static! {
    static ref TICKETS: Arc<Mutex<HashMap<String, Ticket>>> = Arc::new(Mutex::new(HashMap::new()));
    static ref COMMENTS: Arc<Mutex<HashMap<String, Vec<TicketComment>>>> = Arc::new(Mutex::new(HashMap::new()));
}

/// Controller for handling ticket-related routes
pub struct TicketController;

#[derive(Debug, Deserialize, Serialize)]
pub struct NewTicketForm {
    pub title: String,
    pub description: String,
    pub priority: String,
}

#[derive(Debug, Deserialize, Serialize)]
pub struct NewCommentForm {
    pub content: String,
}

#[derive(Debug, Deserialize, Serialize)]
pub struct TicketFilterForm {
    pub status: Option<String>,
    pub priority: Option<String>,
    pub search: Option<String>,
}

impl TicketController {
    /// Helper function to get user from session
    fn get_user_from_session(session: &Session) -> Option<Value> {
        session.get::<String>("user").ok().flatten().and_then(|user_json| {
            serde_json::from_str(&user_json).ok()
        })
    }
    
    /// Lists all tickets with optional filtering
    pub async fn list_tickets(
        session: Session,
        query: web::Query<TicketFilterForm>,
        tmpl: web::Data<Tera>
    ) -> Result<impl Responder> {
        // Get the current user from the session
        let user_value = Self::get_user_from_session(&session);
        let user: Option<User> = user_value.and_then(|v| serde_json::from_value(v).ok());
        
        // If the user is not logged in, redirect to the login page
        if user.is_none() {
            return Ok(HttpResponse::Found()
                .append_header(("Location", "/login"))
                .finish());
        }
        
        let user: User = user.unwrap();
        
        // Get all tickets from the in-memory storage
        let tickets = TICKETS.lock().unwrap();
        
        // Filter tickets based on the query parameters
        let mut filtered_tickets: Vec<Ticket> = tickets.values().cloned().collect();
        
        // Apply status filter if provided
        if let Some(status_str) = &query.status {
            if !status_str.is_empty() {
                let status = match status_str.as_str() {
                    "open" => TicketStatus::Open,
                    "in_progress" => TicketStatus::InProgress,
                    "resolved" => TicketStatus::Resolved,
                    "closed" => TicketStatus::Closed,
                    _ => TicketStatus::Open,
                };
                
                filtered_tickets.retain(|ticket| ticket.status == status);
            }
        }
        
        // Apply priority filter if provided
        if let Some(priority_str) = &query.priority {
            if !priority_str.is_empty() {
                let priority = match priority_str.as_str() {
                    "low" => TicketPriority::Low,
                    "medium" => TicketPriority::Medium,
                    "high" => TicketPriority::High,
                    "urgent" => TicketPriority::Critical,
                    _ => TicketPriority::Medium,
                };
                
                filtered_tickets.retain(|ticket| ticket.priority == priority);
            }
        }
        
        // Regular users can only see their own tickets
        if user.role != crate::models::user::UserRole::Admin {
            filtered_tickets.retain(|ticket| ticket.user_id == user.id.unwrap_or(0));
        }
        
        // Sort tickets by created_at (newest first)
        filtered_tickets.sort_by(|a, b| b.created_at.cmp(&a.created_at));
        
        // Prepare the template context
        let mut ctx = tera::Context::new();
        ctx.insert("active_page", "tickets");
        ctx.insert("tickets", &filtered_tickets);
        ctx.insert("user", &user);
        ctx.insert("filter", &query.into_inner());
        
        // Add filter options for the dropdown menus
        ctx.insert("statuses", &[
            ("", "All Statuses"),
            ("open", "Open"),
            ("in_progress", "In Progress"),
            ("resolved", "Resolved"),
            ("closed", "Closed"),
        ]);
        
        ctx.insert("priorities", &[
            ("", "All Priorities"),
            ("low", "Low"),
            ("medium", "Medium"),
            ("high", "High"),
            ("urgent", "Urgent"),
        ]);
        
        // Render the template
        let rendered = tmpl.render("tickets/list.html", &ctx)
            .map_err(|e| {
                eprintln!("Template rendering error: {}", e);
                actix_web::error::ErrorInternalServerError("Template rendering error")
            })?;
        
        Ok(HttpResponse::Ok().content_type("text/html").body(rendered))
    }
    
    /// Shows the form for creating a new ticket
    pub async fn new_ticket(
        session: Session,
        tmpl: web::Data<Tera>
    ) -> Result<impl Responder> {
        // Get the current user from the session
        let user_value = Self::get_user_from_session(&session);
        let user: Option<User> = user_value.and_then(|v| serde_json::from_value(v).ok());
        
        // If the user is not logged in, redirect to the login page
        if user.is_none() {
            return Ok(HttpResponse::Found()
                .append_header(("Location", "/login"))
                .finish());
        }
        
        let user: User = user.unwrap();
        
        // Prepare the template context
        let mut ctx = tera::Context::new();
        ctx.insert("active_page", "tickets");
        ctx.insert("user", &user);
        
        // Add priority options for the dropdown menu
        ctx.insert("priorities", &[
            ("low", "Low"),
            ("medium", "Medium"),
            ("high", "High"),
            ("urgent", "Urgent"),
        ]);
        
        // Render the template
        let rendered = tmpl.render("tickets/new.html", &ctx)
            .map_err(|e| {
                eprintln!("Template rendering error: {}", e);
                actix_web::error::ErrorInternalServerError("Template rendering error")
            })?;
        
        Ok(HttpResponse::Ok().content_type("text/html").body(rendered))
    }
    
    /// Creates a new ticket
    pub async fn create_ticket(
        session: Session,
        form: web::Form<NewTicketForm>,
        _tmpl: web::Data<Tera>
    ) -> Result<impl Responder> {
        // Get the current user from the session
        let user_value = Self::get_user_from_session(&session);
        let user: Option<User> = user_value.and_then(|v| serde_json::from_value(v).ok());
        
        // If the user is not logged in, redirect to the login page
        if user.is_none() {
            return Ok(HttpResponse::Found()
                .append_header(("Location", "/login"))
                .finish());
        }
        
        let user: User = user.unwrap();
        
        // Parse the priority from the form
        let priority = match form.priority.as_str() {
            "low" => TicketPriority::Low,
            "medium" => TicketPriority::Medium,
            "high" => TicketPriority::High,
            "urgent" => TicketPriority::Critical,
            _ => TicketPriority::Medium,
        };
        
        // Create a new ticket
        let ticket = Ticket::new(
            user.id.unwrap_or(0),
            form.title.clone(),
            form.description.clone(),
            priority
        );
        
        // Add the ticket to the in-memory storage
        let mut tickets = TICKETS.lock().unwrap();
        tickets.insert(ticket.id.clone(), ticket.clone());
        
        // Redirect to the ticket list page
        Ok(HttpResponse::Found()
            .append_header(("Location", "/tickets"))
            .finish())
    }
    
    /// Shows the details of a ticket
    pub async fn show_ticket(
        session: Session,
        path: web::Path<String>,
        tmpl: web::Data<Tera>
    ) -> Result<impl Responder> {
        // Get the current user from the session
        let user_value = Self::get_user_from_session(&session);
        let user: Option<User> = user_value.and_then(|v| serde_json::from_value(v).ok());
        
        // If the user is not logged in, redirect to the login page
        if user.is_none() {
            return Ok(HttpResponse::Found()
                .append_header(("Location", "/login"))
                .finish());
        }
        
        let user: User = user.unwrap();
        
        // Get the ticket ID from the path
        let id = path.into_inner();
        
        // Get the ticket from the in-memory storage
        let tickets = TICKETS.lock().unwrap();
        let ticket = match tickets.get(&id) {
            Some(ticket) => ticket.clone(),
            None => {
                return Ok(HttpResponse::NotFound().finish());
            }
        };
        
        // Regular users can only see their own tickets
        if user.role != crate::models::user::UserRole::Admin && ticket.user_id != user.id.unwrap_or(0) {
            return Ok(HttpResponse::Forbidden().finish());
        }
        
        // Get the comments for this ticket
        let comments = COMMENTS.lock().unwrap();
        let ticket_comments = comments.get(&ticket.id)
            .cloned()
            .unwrap_or_default();
        
        // Prepare the template context
        let mut ctx = tera::Context::new();
        ctx.insert("active_page", "tickets");
        ctx.insert("ticket", &ticket);
        ctx.insert("comments", &ticket_comments);
        ctx.insert("user", &user);
        
        // Add status options for the dropdown menu (for admins)
        ctx.insert("statuses", &[
            ("open", "Open"),
            ("in_progress", "In Progress"),
            ("resolved", "Resolved"),
            ("closed", "Closed"),
        ]);
        
        // Render the template
        let rendered = tmpl.render("tickets/show.html", &ctx)
            .map_err(|e| {
                eprintln!("Template rendering error: {}", e);
                actix_web::error::ErrorInternalServerError("Template rendering error")
            })?;
        
        Ok(HttpResponse::Ok().content_type("text/html").body(rendered))
    }
    
    /// Adds a comment to a ticket
    pub async fn add_comment(
        session: Session,
        path: web::Path<String>,
        form: web::Form<NewCommentForm>
    ) -> Result<impl Responder> {
        // Get the current user from the session
        let user_value = Self::get_user_from_session(&session);
        let user: Option<User> = user_value.and_then(|v| serde_json::from_value(v).ok());
        
        // If the user is not logged in, redirect to the login page
        if user.is_none() {
            return Ok(HttpResponse::Found()
                .append_header(("Location", "/login"))
                .finish());
        }
        
        let user: User = user.unwrap();
        
        // Get the ticket ID from the path
        let id = path.into_inner();
        
        // Get the ticket from the in-memory storage
        let tickets = TICKETS.lock().unwrap();
        let ticket = match tickets.get(&id) {
            Some(ticket) => ticket.clone(),
            None => {
                return Ok(HttpResponse::NotFound().finish());
            }
        };
        
        // Regular users can only comment on their own tickets
        if user.role != crate::models::user::UserRole::Admin && ticket.user_id != user.id.unwrap_or(0) {
            return Ok(HttpResponse::Forbidden().finish());
        }
        
        // Create a new comment
        let comment = TicketComment::new(
            ticket.id.clone(),
            user.id.unwrap_or(0),
            form.content.clone(),
            user.role == crate::models::user::UserRole::Admin
        );
        
        // Add the comment to the in-memory storage
        let mut comments = COMMENTS.lock().unwrap();
        if let Some(ticket_comments) = comments.get_mut(&ticket.id) {
            ticket_comments.push(comment);
        } else {
            comments.insert(ticket.id.clone(), vec![comment]);
        }
        
        // If the ticket is closed, reopen it when a comment is added
        if ticket.status == TicketStatus::Closed {
            let mut tickets = TICKETS.lock().unwrap();
            if let Some(ticket) = tickets.get_mut(&id) {
                ticket.status = TicketStatus::Open;
                ticket.updated_at = chrono::Utc::now();
            }
        }
        
        // Redirect back to the ticket page
        Ok(HttpResponse::Found()
            .append_header(("Location", format!("/tickets/{}", id)))
            .finish())
    }
    
    /// Updates the status of a ticket
    pub async fn update_status(
        session: Session,
        path: web::Path<(String, String)>,
    ) -> Result<impl Responder> {
        // Get the current user from the session
        let user_value = Self::get_user_from_session(&session);
        let user: Option<User> = user_value.and_then(|v| serde_json::from_value(v).ok());
        
        // If the user is not logged in or not an admin, redirect to the login page
        if user.is_none() || user.as_ref().unwrap().role != crate::models::user::UserRole::Admin {
            return Ok(HttpResponse::Found()
                .append_header(("Location", "/login"))
                .finish());
        }
        
        // Get the ticket ID and status from the path
        let (id, status_str) = path.into_inner();
        
        // Parse the status
        let status = match status_str.as_str() {
            "open" => TicketStatus::Open,
            "in_progress" => TicketStatus::InProgress,
            "resolved" => TicketStatus::Resolved,
            "closed" => TicketStatus::Closed,
            _ => TicketStatus::Open,
        };
        
        // Update the ticket status
        let mut tickets = TICKETS.lock().unwrap();
        if let Some(ticket) = tickets.get_mut(&id) {
            ticket.status = status;
            ticket.updated_at = chrono::Utc::now();
        } else {
            return Ok(HttpResponse::NotFound().finish());
        }
        
        // Redirect back to the ticket page
        Ok(HttpResponse::Found()
            .append_header(("Location", format!("/tickets/{}", id)))
            .finish())
    }
    
    /// Shows the user's tickets
    pub async fn my_tickets(
        session: Session,
        tmpl: web::Data<Tera>
    ) -> Result<impl Responder> {
        // Get the current user from the session
        let user_value = Self::get_user_from_session(&session);
        let user: Option<User> = user_value.and_then(|v| serde_json::from_value(v).ok());
        
        // If the user is not logged in, redirect to the login page
        if user.is_none() {
            return Ok(HttpResponse::Found()
                .append_header(("Location", "/login"))
                .finish());
        }
        
        let user: User = user.unwrap();
        
        // Get all tickets from the in-memory storage
        let tickets = TICKETS.lock().unwrap();
        
        // Filter tickets to only show the user's tickets
        let my_tickets: Vec<Ticket> = tickets.values()
            .cloned()
            .filter(|ticket| ticket.user_id == user.id.unwrap_or(0))
            .collect();
        
        // Sort tickets by created_at (newest first)
        let mut sorted_tickets = my_tickets;
        sorted_tickets.sort_by(|a, b| b.created_at.cmp(&a.created_at));
        
        // Prepare the template context
        let mut ctx = tera::Context::new();
        ctx.insert("active_page", "tickets");
        ctx.insert("tickets", &sorted_tickets);
        ctx.insert("user", &user);
        ctx.insert("my_tickets", &true);
        
        // Render the template
        let rendered = tmpl.render("tickets/list.html", &ctx)
            .map_err(|e| {
                eprintln!("Template rendering error: {}", e);
                actix_web::error::ErrorInternalServerError("Template rendering error")
            })?;
        
        Ok(HttpResponse::Ok().content_type("text/html").body(rendered))
    }
}