use actix_web::web;
use actix_session::{SessionMiddleware, storage::CookieSessionStore};
use crate::controllers::home::HomeController;
use crate::controllers::auth::AuthController;
use crate::controllers::ticket::TicketController;
use crate::controllers::calendar::CalendarController;
use crate::middleware::JwtAuth;
use crate::SESSION_KEY;

/// Configures all application routes
pub fn configure_routes(cfg: &mut web::ServiceConfig) {
    // Configure session middleware with the consistent key
    let session_middleware = SessionMiddleware::builder(
        CookieSessionStore::default(),
        SESSION_KEY.clone()
    )
    .cookie_secure(false) // Set to true in production with HTTPS
    .build();

    // Public routes that don't require authentication
    cfg.service(
        web::scope("")
            .wrap(session_middleware)
            // Home routes
            .route("/", web::get().to(HomeController::index))
            .route("/about", web::get().to(HomeController::about))
            .route("/contact", web::get().to(HomeController::contact))
            .route("/contact", web::post().to(HomeController::submit_contact))
            
            // Auth routes
            .route("/login", web::get().to(AuthController::login_page))
            .route("/login", web::post().to(AuthController::login))
            .route("/register", web::get().to(AuthController::register_page))
            .route("/register", web::post().to(AuthController::register))
            .route("/logout", web::get().to(AuthController::logout))
            
            // Protected routes that require authentication
            // These routes will be protected by the JwtAuth middleware in the main.rs file
            .route("/editor", web::get().to(HomeController::editor))
            
            // Ticket routes
            .route("/tickets", web::get().to(TicketController::list_tickets))
            .route("/tickets/new", web::get().to(TicketController::new_ticket))
            .route("/tickets", web::post().to(TicketController::create_ticket))
            .route("/tickets/{id}", web::get().to(TicketController::show_ticket))
            .route("/tickets/{id}/comment", web::post().to(TicketController::add_comment))
            .route("/tickets/{id}/status/{status}", web::post().to(TicketController::update_status))
            .route("/my-tickets", web::get().to(TicketController::my_tickets))
            
            // Calendar routes
            .route("/calendar", web::get().to(CalendarController::calendar))
            .route("/calendar/events/new", web::get().to(CalendarController::new_event))
            .route("/calendar/events", web::post().to(CalendarController::create_event))
            .route("/calendar/events/{id}/delete", web::post().to(CalendarController::delete_event))
    );
    
    // Keep the /protected scope for any future routes that should be under that path
    cfg.service(
        web::scope("/protected")
            .wrap(JwtAuth)  // Apply JWT authentication middleware
    );
}